import numpy as np

from gpaw.xc.lda import LDA, PurePythonLDAKernel, lda_x, lda_c
from gpaw.xc.gga import gga_constants, GGA, PurePythonGGAKernel, \
    gga_x as gpawgga_x, gga_c as gpawgga_c
from gpaw.xc.lda import G

from cmplx.cslib import diff, stitch, rootbranch, cuberoot, \
    sixthroot, logarithm, epsx, epsc, epsx_complex, epsc_complex


x = np.linspace(-6, 6, 2001)
dx = x[1] - x[0]
theta = 0.3
phase = np.exp(1j * theta)
z = x * phase
n = np.exp(-x**2)
nt = np.exp(1j * theta) * np.exp(-z**2)

gradn = np.gradient(n, dx)
gradnt = np.gradient(nt, dx) / phase
#gradn2 = np.gradient(n, 2)

#import pylab as pl

#pl.plot(x, gradn, 'o')
#pl.plot(x[:-1] + dx / 2, (n[1:] - n[:-1]) / dx, 'r.')
#pl.plot(x, gradn2)
#pl.show()

print 'electrons', n.sum() * dx, nt.sum() * dx

#n = 0.5
#gradn = 0.3
n_g = n #* np.ones(1)
gradn_g = gradn #* np.ones(1)

n_sg = n_g.reshape(1, -1)
gradn_sg = gradn_g.reshape(1, -1)

#exggagpaw = np.array([[0.0]])
#vxggagpaw = np.array([[0.0]])

def zeros():
    return np.zeros(len(n))

vxgpaw_g = zeros()
exgpaw_g = zeros()
ecgpaw_g = zeros()
vcgpaw_g = zeros()

excgpaw_g = zeros()
vxcgpaw_sg = zeros().reshape(1, -1)

lda_x(0, exgpaw_g, n_g, vxgpaw_g)
lda_c(0, ecgpaw_g, n_g, vcgpaw_g, np.array(0.0))

ex, vx = epsx(n_g)
ec, vc = epsc(n_g)

ext, vxt = epsx_complex(nt, theta)
ect, vct = epsc_complex(nt, theta)


def int(arr):
    return arr.sum() * dx

#print 'n gradn  %f %f' % (n_g, gradn_g)
print 'ex    %8f %8f %s' % (int(exgpaw_g), int(ex * n_g), int(ext * nt))
print 'ec    %8f %8f %s' % (int(ecgpaw_g), int(ec * n_g), int(ect * nt))
print 'exc   %8f %8f %s' % (int(exgpaw_g + ecgpaw_g), int((ex + ec) * n_g),
                            int((ext + ect) * nt))
print 'vx n  %8f %8f %s' % (int(vxgpaw_g * n), int(vx * n), int(vxt * nt))
print 'vc n  %8f %8f %s' % (int(vcgpaw_g * n), int(vc * n), int(vct * nt))
print 'vxc n %8f %8f %s' % (int((vxgpaw_g + vcgpaw_g) * n), 
                            int((vx + vc) * n),
                            int((vxt + vct) * nt))


ldaobj = LDA(PurePythonLDAKernel())
ldaobj.calculate_lda(excgpaw_g, n_sg, vxcgpaw_sg)
print 'lda xc gpaw e::v', int(excgpaw_g), int(vxcgpaw_sg)

print
print 'begin gga'

excgpawgga_g = zeros()
vxcgpawgga_sg = zeros().reshape(1, -1)
sigma_sg = (gradn**2).reshape(1, -1)
dedsigma_sg = zeros().reshape(1, -1)



ggaobj = GGA(PurePythonGGAKernel('pyPBE'))
ggaobj.calculate_gga(excgpawgga_g, n_sg, vxcgpawgga_sg, sigma_sg,
                     dedsigma_sg)

def gga_x(n, gradn):

    # XXX
    kappa = 0.804
    mu = 0.2195149727645171
    #beta = 0.06672455060314922


    C0I, C1, C2, C3, CC1, CC2, IF2, GAMMA = gga_constants()

    if n.dtype == complex:
        n = n * np.exp(-1j * theta)
        rs = stitch(C0I / (n + 1e-40), cuberoot)
        gradn = gradn / phase
    else:
        rs = (C0I / n)**(1 / 3.)

    # lda part
    ex = C1 / rs
    dexdrs = -ex / rs

    #ex11 = C1 / rs * (1 + kappa - 
    #                  kappa / (1 + mu * (C2 * rs / n)**2 
    #                           * gradn**2))
    
    #print 'grumble', (ex11 * n).sum() * dx

    #if n.dtype == complex:
    #    gradn = gradn * np.exp(-1j * theta)

    a2 = gradn**2

    # gga part
    c = (C2 * rs / n)**2
    s2 = a2 * c
    
    x = 1.0 + mu * s2 / kappa
    Fx = 1.0 + kappa - kappa / x
    
    #if n.dtype == complex:
    #    import pylab as pl
    #    pl.plot(np.abs(1/x))
    #    #pl.plot(x.imag)
    #    pl.show()
    #    raise SystemExit

    #dFxds2 = mu / x**2
    #ds2drs = 8.0 * c * a2 / rs
    #dexdrs = dexdrs * Fx + ex * dFxds2 * ds2drs
    #dexda2 = ex * dFxds2 * c
    ex *= Fx
    
    #if n.dtype == complex:
    #    import pylab as pl
    #    pl.plot(ex.real)
    #    pl.plot(ex.imag)
    #    pl.show()
    #    raise SystemExit

    vx = ex - rs * dexdrs / 3.
    return ex, vx

def gga_c(n, gradn):
    from gpaw.xc.lda import G

    BETA = 0.06672455060314922

    C0I, C1, C2, C3, CC1, CC2, IF2, GAMMA = gga_constants()

    if n.dtype == complex:
        n = n * np.exp(-1j * theta)
        rs = stitch(C0I / (n + 1e-40), cuberoot)
        gradn = gradn / phase
    else:
        rs = (C0I / n)**(1 / 3.)
    
    # lda part
    ec, decdrs_0 = G(rs**0.5, 0.031091, 0.21370, 7.5957,
                     3.5876, 1.6382, 0.49294)

    decdrs = decdrs_0

    # gga part
    n2 = n**2
    a2 = gradn**2
    t2 = C3 * a2 * rs / n2
    y = -ec / GAMMA

    x = np.exp(y)

    A = np.zeros_like(x)
    indices = np.nonzero(y)
    A[indices] = (BETA / (GAMMA * (x[indices] - 1.0)))

    At2 = A * t2
    nom = 1.0 + At2
    denom = nom + At2 * At2
    H = GAMMA * np.log(1.0 + BETA * t2 * nom / (denom * GAMMA))
    #tmp = (GAMMA * BETA / (denom * (BETA * t2 * nom + GAMMA * denom)))
    #tmp2 = A * A * x / BETA
    #dAdrs = tmp2 * decdrs
    #dHdt2 = (1.0 + 2.0 * At2) * tmp
    #dHdA = -At2 * t2 * t2 * (2.0 + At2) * tmp
    #decdrs += dHdt2 * 7.0 * t2 / rs + dHdA * dAdrs
    #decda2 = dHdt2 * C3 * rs / n2
    ec += H

    vc = ec - rs * decdrs / 3.
    return ec, vc #rs, decdrs, decda2



ex, vx = gga_x(n, gradn)
ext, vxt = gga_x(nt, gradnt)

ex1, rs1, dexdrs1, dexda21 = gpawgga_x('PBE', 0, n_g, sigma_sg[0], .804,
                                       0.2195149727645171)

ec, vc = gga_c(n_g, gradn)
ect, vct = gga_c(nt, gradnt)

ec1, rs1, decdrs1, decda21, decdzeta1 = gpawgga_c('PBE', 0, n_g, sigma_sg[0],
                                                  None, 0.06672455060314922)
                                                  

print 'ex gga %8f %8f %8s' % (int(ex1 * n), int(ex * n), int(ext * nt))
print 'ec gga %8f %8f %8s' % (int(ec1 * n), int(ec * n), int(ect * nt))

print 'vx gga %8f %8s' % (int(vx * n), int(vxt * nt))
print 'vc gga %8f %8s' % (int(vc * n), int(vct * nt))
print 'vxc gga %8f combined' % int((vx + vc) * n)

print 'exc gga %8f %8f' % (int(excgpawgga_g), int((ex + ec) * n))
print 'vxc gga %8f %8f' % (int(vxcgpawgga_sg[0] * n), int((vx + vc) * n))
print 'exc gga combined %8f %8f' % (int((ex1 + ec1) * n), int((ex + ec) * n))


