from optparse import OptionParser

import numpy as np
import pylab as pl

from pydft.grid import Grid
from pydft.operator import get_kinetic
from pydft.poisson import SoftCoulombPoisson
from pydft.eigen import CSEigenSolver
from pydft.cscaling import CLDA
from pydft.xc import LDA

def build_parser():
    p = OptionParser()
    p.add_option('--theta', default=np.pi / 8.0, type=float,
                 help='complex scaling angle [default=pi/8]')
    return p
parser = build_parser()
opts, args = parser.parse_args()
theta = opts.theta

np.set_printoptions(precision=8, suppress=1)

nstates = 5
ne = 2

def phase(N):
    return np.exp(1j * N * theta)

x = np.linspace(-6, 6, 201)
z = x * phase(1)

g = Grid(x)
poisson = SoftCoulombPoisson(g, softness=phase(-2))
eig = CSEigenSolver(g, nstates)

T_gg = get_kinetic(g.x, order=4) * phase(-2)
vext_g = 2.0 * z**2

vH_g = g.zeros(dtype=complex)
n_g = g.zeros(dtype=complex)
vxc_g = g.zeros(dtype=complex)

f_n = np.zeros(nstates * 2)
f_n[:ne] = 1.0
f_n = f_n.reshape(-1, 2).sum(axis=-1)

mix = 1.0

xc = CLDA(theta)

for i in range(30):
    v_g = vH_g + vxc_g + vext_g
    H_gg = T_gg.copy() + np.diag(v_g)

    eps_n, psi_gn = eig.solve(H_gg)

    n_prev = n_g
    n_g = (psi_gn[:, :]**2 * f_n[None, :]).sum(axis=1)
    logerr = np.log10(abs(n_prev - n_g).sum() * (g.dx / ne))
    print 'iter %4d :: logerr %f' % (i, logerr)
    if logerr < -8:
        break
    
    n_g = n_g * mix + n_prev * (1.0 - mix)
    vH_g = poisson.solve(n_g) * phase(-1)
    exc, vxc_g = xc.xc(n_g)


Ekin = (eps_n[:nstates] * f_n).sum() - g.int(v_g * n_g)

Eext = g.int(n_g * vext_g)
Ehartree = 0.5 * g.int(n_g * vH_g)
#Exc = g.int(exc * n_g)

Etot = Ekin + Ehartree + Eext # XC

print
print 'Eigs'
print 'Re', eps_n.real
print 'Im', eps_n.imag
print
print 'Ekin', Ekin
print 'EHa', Ehartree
#print 'Exc  %12.8f %12.8f' % Exc
print 'Eext', Eext
print 'Etot', Etot


#nt = phase(1) * np.exp(-z**2)



#vt = phase(-1) * poisson.solve(nt)

#pl.plot(x, vt.real)
#pl.plot(x, vt.imag)

#print (nt * vt).sum() * g.dx

#pl.show()
