import numpy as np

from pydft.grid import Grid
from pydft.eigen import EigenSolver
from pydft.operator import get_kinetic, get_derivative
from pydft.poisson import SoftCoulombPoisson
from pydft.util import dot
from pydft.xc import LDA, PBE, NoXC

g = Grid(np.linspace(-5.95, 5.95, 201))
p = SoftCoulombPoisson(g)

T_gg = get_kinetic(g.x, order=6)
vext_g = 2.0 * g.x**2

vH_g = g.zeros()
n_g = g.zeros()
vxc_g = g.zeros()

nstates = 6
ne = 2

f_n = np.zeros(nstates * 2)
f_n[:ne] = 1.0
f_n = f_n.reshape(-1, 2).sum(axis=-1)

mix = 1.0

xc = PBE(g)
#xc = LDA()
#xc = NoXC()

eig = EigenSolver(g, nstates)

for i in range(30):
    v_g = vH_g + vxc_g + vext_g
    H_gg = T_gg.copy() + np.diag(v_g)

    eps_n, psi_gn = eig.solve(H_gg)

    n_prev = n_g
    n_g = (psi_gn[:, :]**2 * f_n[None, :]).sum(axis=1)
    logerr = np.log10(abs(n_prev - n_g).sum() * (g.dx / ne))
    print 'iter %4d :: logerr %f' % (i, logerr)
    if logerr < -8:
        break
    
    n_g = n_g * mix + n_prev * (1.0 - mix)
    vH_g = p.solve(n_g) # XXX
    exc, vxc_g = xc.xc(n_g)
    

print 'eps', eps_n[:nstates]

Ekin = (eps_n[:nstates] * f_n).sum() - g.int(v_g * n_g)

Eext = g.int(n_g * vext_g)
Ehartree = 0.5 * g.int(n_g * vH_g)
Exc = g.int(exc * n_g)

Etot = Ekin + Ehartree + Exc + Eext

print 'Ekin %12.8f' % Ekin
print 'EHa  %12.8f' % Ehartree
print 'Exc  %12.8f' % Exc
print 'Eext %12.8f' % Eext
print 'Etot %12.8f' % Etot


if 0:
    import pylab as pl
    fig = pl.figure()
    ax1 = fig.add_subplot(221)
    ax2 = fig.add_subplot(222)
    ax3 = fig.add_subplot(223)

    ax1.plot(g.x, n_g)
    for n in range(nstates):
        ax2.plot(g.x, psi_gn[:, n])

    ax3.plot(g.x, vH_g)


    pl.show()
