import numpy as np

class EigenSolver:
    def __init__(self, grid, nstates=None):
        self.grid = grid
        self.nstates = nstates
    
    def solve(self, H_gg):
        eps_n, psi_gn = np.linalg.eigh(H_gg)
        nstates = self.nstates
        if nstates is None:
            nstates = len(H_gg)
        eps_n = eps_n[:nstates].copy()
        psi_gn = psi_gn[:, :nstates].copy()
        norms = self.grid.norm(psi_gn, axis=0)
        psi_gn /= norms[None, :]
        return eps_n, psi_gn


class CSEigenSolver:
    def __init__(self, grid, nstates=None):
        self.grid = grid
        self.nstates = nstates

    def solve(self, H_gg):
        eps_n, psi_gn = np.linalg.eig(H_gg)
        nstates = self.nstates
        if nstates is None:
            nstates = len(H_gg)
        
        args = np.argsort(eps_n.real)
        eps_n = eps_n[args[:nstates]].copy()
        psi_gn = psi_gn[:, args[:nstates]].copy()
        normsqrs = (psi_gn**2).sum(axis=0) * self.grid.dx
        psi_gn[:, :] /= np.sqrt(normsqrs)[None, :]
        return eps_n, psi_gn

