import numpy as np


lstencils = [[0.],
             [-2., 1.],
             [-5./2., 4./3., -1./12.],
             [-49./18., 3./2., -3./20., 1./90.],
             [-205./72., 8./5., -1./5., 8./315., -1./560.],
             [-5269./1800., 5./3., -5./21., 5./126., -5./1008., 1./3150.],
             [-5369./1800., 12./7., -15./56., 10./189., -1./112., 2./1925.,
              -1./16632.]]

deriv_stencils = [np.array(coefs) / normalization
                  for coefs, normalization
                  in [([0.], 1.), 
                      ([0., 1.], 2.),
                      ([0., 8., -1.], 12.,),
                      ([0., 45., -9., 1.], 60.)]]


def get_derivative(x, order=1):
    dx = x[1] - x[0]
    N = len(x)

    stencil = deriv_stencils[order] / dx

    D = np.zeros((N, N))
    Dflat = D.ravel()
    for i, val in enumerate(stencil):
        Dflat[i::N + 1] = val
        Dflat[i * N::N + 1] = -val
    return D


def get_kinetic(x, order=1, periodic=False):
    N = len(x)
    dx = x[1] - x[0]
    T = np.zeros((N, N))
    Tflat = T.ravel()

    stencil = lstencils[order]

    for i, val in enumerate(stencil):
        Tflat[i::N + 1] = val
        Tflat[i * N::N + 1] = val
    if periodic:
        if order > 1:
            raise NotImplementedError
        T[-1, 0] = 1.0
        T[0, -1] = 1.0
    T *= -0.5 / dx**2
    return T

