import numpy as np

class SoftCoulombPoisson:
    def __init__(self, grid, softness=1.0):
        self.grid = grid
        self.softness = softness

    def solve(self, n_g):
        grid = self.grid
        softness = self.softness
        v_g = grid.zeros(dtype=n_g.dtype)
        x = grid.x
        for i, xp in enumerate(x):
            v_g[i] = (n_g / np.sqrt((x - xp)**2 + softness)).sum()
        v_g *= grid.dx
        return v_g

class RadialPoisson:
    def __init__(self, grid):
        self.grid = grid
        from gpaw.utilities import hartree
        self.hartree = hartree

    def solve(self, n_g):
        g = self.grid
        dp = n_g * g.x * g.dx
        dq = dp * g.x
        p = np.add.accumulate(dp[::-1])[::-1]
        q = np.add.accumulate(dq[::-1])[::-1]
        v_g = 4 * np.pi * (p - 0.5 * dp - (q - 0.5 * dq - q[0]) / g.x)
        return v_g
