import numpy as np

from pydft.grid import Grid, RadialGrid
from pydft.eigen import EigenSolver
from pydft.operator import get_kinetic, get_derivative
from pydft.poisson import SoftCoulombPoisson, RadialPoisson
from pydft.util import dot
from pydft.xc import LDA, PBE, NoXC

g = RadialGrid(np.linspace(0, 9, 301)[1:])
#poisson = SoftCoulombPoisson(g)
poisson = RadialPoisson(g)

T_gg = get_kinetic(g.x, order=6)
#vext_g = 2.0 * g.x**2
#vext_g = g.x * 0.0 #-1.0 / g.x
vext_g = -1.0 / g.x

vH_g = g.zeros()
r2n_g = g.zeros()
n_g = g.zeros()
vxc_g = g.zeros()

nstates = 2
ne = 1

f_n = np.zeros(nstates * 2)
f_n[:ne] = 1.0
f_n = f_n.reshape(-1, 2).sum(axis=-1)

mix = 0.8

#xc = PBE(g)
xc = LDA()
#xc = NoXC()

eig = EigenSolver(g, nstates)

for i in range(30):
    v_g = vH_g + vxc_g + vext_g
    v_g /= np.sqrt(4 * np.pi)
    H_gg = T_gg.copy() + np.diag(v_g)

    eps_n, rpsi_gn = eig.solve(H_gg)#eigensolve(g, H_gg, nstates)

    n_prev = r2n_g
    r2n_g = (rpsi_gn[:, :]**2 * f_n[None, :]).sum(axis=1)
    #logerr = np.log10(g.integrate(np.abs(n_prev - r2n_g)) / ne)
    logerr = np.log10(abs(n_prev - r2n_g).sum() * (g.dx / ne))
    print 'iter %4d :: logerr %f' % (i, logerr)
    if logerr < -8:
        break

    r2n_g = r2n_g * mix + n_prev * (1.0 - mix)
    n_g = r2n_g / g.x**2
    #vH_g = poisson.solve(n_g)# XXX
    vH_g = np.zeros_like(n_g)
    rho_g = n_g.copy()
    #rho_g[0] -= 1.0 / (g.x[0]**2 * g.dx)
    #print 'int rho', (rho_g * g.x**2).sum() * g.dx
    #print 'charge1', g.int(rho_g)
    #rho_g[0] = -1.0 / g.dx
    #print 'charge', g.int(rho_g)
    vH_g = poisson.solve(rho_g) / (4. * np.pi)# XXX
    exc, vxc_g = xc.xc(n_g)
    #vxc_g /= (4.0 * np.pi)**.5
    #exc *= 0
    #vxc_g *= 0

print 'eps', eps_n

Ekin = (eps_n[:nstates] * f_n).sum() - g.int(v_g * n_g)

Eext = g.int(n_g * vext_g)
Ehartree = 0.5 * g.int(n_g * vH_g)
Exc = g.int(exc * n_g)

Etot = Ekin + Ehartree + Exc + Eext

print 'Eband %8f' % (eps_n * f_n).sum()

print 'Ekin %8f' % Ekin
print 'EHa  %8f' % Ehartree
print 'Exc  %8f' % Exc
print 'Eext %8f' % Eext
print 'Etot %8f' % Etot


if 1:
    import pylab as pl
    fig = pl.figure()
    ax1 = fig.add_subplot(221)
    ax2 = fig.add_subplot(222)
    ax3 = fig.add_subplot(223)

    ax1.plot(g.x, n_g)
    for n in range(nstates):
        ax2.plot(g.x, rpsi_gn[:, n] / g.x)
    #ax2.set_yscale('log')

    ax3.plot(g.x, vH_g, 'b', label='vH')
    ax3.plot(g.x, vext_g, 'g', label='vext')
    ax3.plot(g.x, vxc_g, 'c', label='vxc')
    ax3.plot(g.x, v_g, 'r', label='v')
    ax3.legend(loc='best')
    pl.show()
